package cn.zhangdeman.context;

import cn.zhangdeman.consts.RecordField;
import cn.zhangdeman.exception.CustomException;
import cn.zhangdeman.logger.LoggerInstance;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpServletResponse;
import lombok.Getter;
import lombok.Setter;
import org.springframework.http.HttpStatus;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

// 响应数据
@Getter
@Setter
@JsonIgnoreProperties(ignoreUnknown = true) // 忽略未知属性字段
public class Response<Data> implements Serializable {
    @JsonProperty(RecordField.REQUEST_ID)
    private String requestId; // 本次请求request id
    @JsonProperty(RecordField.TRACE_ID)
    private String traceId; // 全局trace id
    @JsonProperty(RecordField.COST)
    private Long cost; // 请求耗时
    @JsonProperty(RecordField.RESPONSE_CODE)
    private String code; // 响应业务状态码
    @JsonProperty(RecordField.RESPONSE_MESSAGE)
    private String message; // 响应消息
    @JsonProperty(RecordField.RESPONSE_DATA)
    private Data data; // 响应数据
    @JsonProperty(RecordField.RESPONSE_ERROR_DETAIL)
    private Object errorDetail; // 异常详情
    @JsonIgnore
    private HttpStatus httpStatus = HttpStatus.OK; // 响应http状态码, 默认 200
    @JsonProperty(RecordField.RESPONSE_HTTP_CODE)
    private Integer httpCode = HttpStatus.OK.value(); // 响应http状态码, 默认 200
    @JsonProperty(RecordField.RESPONSE_HEADER)
    private Map<String, String> header = new HashMap<>(); // 响应头
    @JsonProperty(RecordField.RESPONSE_COOKIE)
    private Map<String, String> cookie = new HashMap<>(); // 响应cookie
    @JsonIgnore
    private String contentType = "application/json;charset=utf-8";
    @JsonIgnore
    private RuntimeContext runtimeContext;
    @JsonIgnore
    private Boolean handleSuccess; // 是否处理成功响应

    // 响应实例
    public Response(RuntimeContext runtimeContext) {
        this.runtimeContext = runtimeContext;
        setTraceId(runtimeContext.getTraceId());
        setRequestId(runtimeContext.getRequestId());
        setCost(System.currentTimeMillis() - runtimeContext.getStartTimeStamp()); // 耗时, 毫秒
    }

    public void addHeader(String key, String value) {
        if (null == this.header) {
            this.header = new HashMap<>();
        }
        this.header.put(key, value);
    }

    public void addCookie(String key, String value) {
        if (null == this.cookie) {
            this.cookie = new HashMap<>();
        }
        this.cookie.put(key, value);
    }

    // 设置响应的header与cookie
    private void setResponseHeaderAndCookie() {
        // 设置header
        if (null != getHeader()) {
            // 处理header
            for (Map.Entry<String, String> headerInfo: getHeader().entrySet()) {
                runtimeContext.getHttpServletResponse().addHeader(headerInfo.getKey(), headerInfo.getValue());
            }
        }
        // 设置cookie
        if (null != getCookie()) {
            // 处理header
            for (Map.Entry<String, String> cookieInfo: getCookie().entrySet()) {
                runtimeContext.getHttpServletResponse().addCookie(new Cookie(cookieInfo.getKey(), cookieInfo.getValue()));
            }
        }
        // 设置响应状态码
        runtimeContext.getHttpServletResponse().setStatus(getHttpCode());
    }
    
    // 请求成功, 200 状态码
    public Response<Data> success(RuntimeContext runtimeContext, Data data) {
        return success(runtimeContext, HttpStatus.OK, data);
    }

    // 请求成功, 自定义http状态码
    public Response<Data> success(RuntimeContext runtimeContext, HttpStatus httpStatus, Data data) {
        setHandleSuccess(true);
        if (null == getCode()) {
            setCode("0");
        }
        if (null == message) {
            setMessage("请求成功");
        }
        return any(runtimeContext, httpStatus, getCode(), getMessage(), data);
    }

    // 请求失败, 返回200
    public Response<Data> failure(RuntimeContext runtimeContext, String code, String message, Object errorDetail) {
        return failure(runtimeContext, HttpStatus.OK, code, message, errorDetail);
    }

    public Response<Data> failure(RuntimeContext runtimeContext, HttpStatus httpStatus, String code, String message, Object errorDetail) {
        setHandleSuccess(false);
        setErrorDetail(errorDetail);
        return any(runtimeContext, httpStatus, code, message, null);
    }

    public Response<Data> failure(RuntimeContext runtimeContext, CustomException customException) {
        return failure(runtimeContext, HttpStatus.OK, customException);
    }

    public Response<Data> failure(RuntimeContext runtimeContext, HttpStatus httpStatus, CustomException customException) {
        setHandleSuccess(false);
        setErrorDetail(customException.getStackTrace()); // 异常堆栈
        // 错误码使用 category+_+code
        return any(runtimeContext, httpStatus, customException.getCategory() + "_"+customException.getCode(), customException.getMessage(), null);
    }

    // 任意姿势的响应
    public Response<Data> any(RuntimeContext runtimeContext, HttpStatus httpStatus, String code, String message, Data data) {
        this.runtimeContext = runtimeContext;
        setHttpStatus(httpStatus);
        addHeader("Content-Type", getContentType()); // header追加响应类型
        setHttpCode(httpStatus.value());
        setCode(code);
        setMessage(message);
        setData(data);
        setResponseHeaderAndCookie();
        this.runtimeContext.setResponse((Response<Object>)this);
        // 记录日志
        if (getHandleSuccess()) {
            LoggerInstance.REQUEST_OUTPUT_LOGGER.info("接口处理成功");
        } else {
            LoggerInstance.REQUEST_OUTPUT_LOGGER.error("接口处理失败");
        }
        return this;
    }
    // 序列化
    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }
}
